package net.osmand.data;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import net.osmand.osm.LatLon;

public class ESBRData implements Serializable, Comparable {
	private long timeStamp;
	private long nodeID;
	private int msgID;
	private long streetID;
	private boolean warning;
	private LatLon[] route;
	
	public ESBRData(long nodeID,  int msgID, long streetID, boolean warning, LatLon[] route) {
		this.timeStamp=System.currentTimeMillis();
		this.nodeID = nodeID;
		this.streetID = streetID;
		this.msgID = msgID;
		this.warning = warning;
		this.route = route;
	}
	
	@Override
	public boolean equals(Object myData){
		if(myData==this)
			return true;
		
		if(myData==null)
			return false;
		
		if ( !(myData instanceof ESBRData) ) return false;
		
		if( timeStamp!=((ESBRData)myData).timeStamp || 
			nodeID != ((ESBRData)myData).getNodeID() || 
			msgID != ((ESBRData)myData).getMsgID())
		{
			return false;
		}	
		
		return true;
	}
	
	@Override
	public int hashCode(){
		return new String(Long.toString(nodeID)+Long.toString(msgID)).hashCode();
	}
	
	public int getMsgID() {
		return msgID;
	}

	public void setMsgID(int msgID) {
		this.msgID = msgID;
	}

	public long getNodeID() {
		return nodeID;
	}
	
	public void setNodeID(int nodeID) {
		this.nodeID = nodeID;
	}
	
	public long getStreetID() {
		return streetID;
	}
	
	public void setStreetID(long streetID) {
		this.streetID = streetID;
	}

	public LatLon[] getRoute() {
		return route;
	}

	public void setRoute(LatLon[] route) {
		this.route = route;
	}

	public boolean isWarning() {
		return warning;
	}

	public void setWarning(boolean warning) {
		this.warning = warning;
	}
	
	private void writeObject(ObjectOutputStream out) throws IOException{
		out.defaultWriteObject();
	}
	
	private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException{			
		in.defaultReadObject();
	}

	
	@Override
	public int compareTo(Object another) {
		
		if(this.equals(another)){
			return 0;
		}
		else if((this.timeStamp-((ESBRData)another).timeStamp)!=0){
			return Long.signum(this.timeStamp-((ESBRData)another).timeStamp);		
		}
		else if((this.nodeID-((ESBRData)another).nodeID)!=0){
			return Long.signum(this.nodeID-((ESBRData)another).nodeID);		
		}
		else {
			return this.msgID-((ESBRData)another).msgID;
		}
	}
	
}
