package net.osmand.plus.geoassist;


import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Environment;
import android.os.Looper;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.Math;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;


import net.osmand.binary.BinaryMapDataObject;
import net.osmand.binary.BinaryMapIndexReader;
import net.osmand.osm.LatLon;
import net.osmand.osm.MapUtils;
import net.osmand.plus.OsmandApplication;
import net.osmand.plus.activities.ApplicationMode;
import net.osmand.plus.routing.RouteProvider.RouteCalculationResult;
import net.osmand.plus.routing.RouteProvider.RouteService;
import net.osmand.plus.routing.RouteProvider;
import net.osmand.plus.routing.RoutingHelper;
import net.osmand.router.BinaryRoutePlanner;
import net.osmand.router.CarRouter;
import net.osmand.router.RoutingContext;
import net.osmand.router.BinaryRoutePlanner.RouteSegment;

public class GeoHelper extends Thread implements LocationListener{
	
	private RoutingHelper router;
	private LocationManager location;
	private volatile Location lastFix;
	private volatile Location lastFixOnStreet;		
		
	private Location lastSegmentLocation; // New street is only calculated every 20 meters.
	private volatile float bearing=0;
	private volatile float bearingOnStr=0;
	private volatile float speed=0;
	private volatile float speedOnStr=0;
	
	private volatile  RouteSegment mySegment=null;
	private OsmandApplication app;
	private ApplicationMode mode=ApplicationMode.CAR;
	
	final static double SEGMENT_UPDATE_DIST = 0;  
	final static boolean LOGGING = true;
	private PrintWriter pw;


	public float getBearing() {
		return bearing;
	}

	public float getSpeed() {
		return speed;
	}
	
	
	//TODO SOLO COMPRUEBA EL PRIMER TROZO !! !! 
	public int[] posOnSegment(Location location, RouteSegment mySegment)
	{
		int px = MapUtils.get31TileNumberX(location.getLongitude());
		int py = MapUtils.get31TileNumberY(location.getLatitude());
		
		BinaryMapDataObject r = mySegment.getRoad();
		int x=0,y=0;
		double prevProj=0;
	
		//Get first segment distance
		double mDist = BinaryRoutePlanner.squareRootDist(
				r.getPoint31XTile(mySegment.getSegmentStart()+1), r.getPoint31YTile(mySegment.getSegmentStart()+1),
				r.getPoint31XTile(mySegment.getSegmentStart()), r.getPoint31YTile(mySegment.getSegmentStart()));
		double projection = BinaryRoutePlanner.calculateProjection(r.getPoint31XTile(mySegment.getSegmentStart()), r.getPoint31YTile(mySegment.getSegmentStart()), r.getPoint31XTile(mySegment.getSegmentStart()+1), r.getPoint31YTile(mySegment.getSegmentStart()+1),
				px, py, mDist);
		
		if( projection < 0 && prevProj >=0 )
		{
			x=r.getPoint31XTile(mySegment.getSegmentStart());
			y=r.getPoint31YTile(mySegment.getSegmentStart());
		}
		else if(projection > mDist)
		{
			x=r.getPoint31XTile(mySegment.getSegmentStart()+1);
			y=r.getPoint31YTile(mySegment.getSegmentStart()+1);
		}
		else {
			double[] vector= new double[2];
		
			vector[0]=(r.getPoint31XTile(mySegment.getSegmentStart()+1)-r.getPoint31XTile(mySegment.getSegmentStart())) /mDist;
			vector[1]=(r.getPoint31YTile(mySegment.getSegmentStart()+1)-r.getPoint31YTile(mySegment.getSegmentStart())) /mDist;
		
			x=(int)(r.getPoint31XTile(mySegment.getSegmentStart())+vector[0]*projection);
			y=(int)(r.getPoint31YTile(mySegment.getSegmentStart())+vector[1]*projection);
		}
		prevProj = projection;
		
		return new int[] {x,y};
	}
	
	public GeoHelper(RoutingHelper router, LocationManager location, OsmandApplication app) {
		super();
		this.router=router;		
		this.location=location;
		this.app=app;
	}
	
	public RouteSegment findSegment(Location loc){
		try{
			BinaryMapIndexReader[] files = app.getResourceManager().getRoutingMapFiles();
			BinaryRoutePlanner router = new BinaryRoutePlanner(files);
			RoutingContext ctx = new RoutingContext();
			ctx.setUsingShortestWay(false);
			ctx.setRouter(new CarRouter());
			ctx.setUseStrategyOfIncreasingRoadPriorities(true);
			ctx.setUseDynamicRoadPrioritising(true);
			RouteSegment segment=router.findRouteSegment(loc.getLatitude(), loc.getLongitude(), ctx);
			return segment;
		}
		catch (IOException e) {
			System.out.println(e.toString());
		}
		catch (Exception e) {
			System.out.println(e.toString());
		}
		return null;
	}
	
	@Override
	public void onProviderDisabled(String provider) {
	}
	
	@Override
	public void onProviderEnabled(String provider) {
	}
	
	@Override
	public void onStatusChanged(String provider, int status, Bundle extras) {
	}

	@Override
	public void onLocationChanged(Location location) {
		float [] result= new float[3];
		String msg= new String(location.toString());
		logMessage(msg, "Fix");
		if(lastSegmentLocation!= null)
		{
			double distance=lastSegmentLocation.distanceTo(location);
		
			if(distance > SEGMENT_UPDATE_DIST || mySegment==null)
			{
				updateSegment(location);
			}
		}
		else
		{
			updateSegment(location);
		}
		
		int [] loc=new int[] {0,0};
		
		if(mySegment!=null){
			loc= posOnSegment(location, mySegment);
		}
		
		
		
		Location locOnStreet = new Location("GeoHelper");
		
		locOnStreet.setLongitude(MapUtils.get31LongitudeX(loc[0]));
		locOnStreet.setLatitude(MapUtils.get31LatitudeY(loc[1]));
		
		if(lastFix!=null && lastFixOnStreet != null)
		{
			float delay=0;

			Location.distanceBetween(lastFix.getLatitude(), lastFix.getLongitude(), location.getLatitude(), location.getLongitude(), result);
			delay = ((location.getTime()-lastFix.getTime())/1000);
			speed= result[0]/delay;
			bearing=result[1];
			
			Location.distanceBetween(lastFixOnStreet.getLatitude(), lastFixOnStreet.getLongitude(), locOnStreet.getLatitude(), locOnStreet.getLongitude(), result);
			speedOnStr= result[0]/delay;
			bearingOnStr=result[1];
			msg = new String (""+delay);
			logMessage(msg, "FixDelay");
		}
		
		
		locOnStreet.setTime(location.getTime());
		lastFixOnStreet=locOnStreet;
		lastFix=location;
		msg= new String(lastFix.getLatitude()+" "+lastFix.getLongitude()+"\t OnStreet "+lastFixOnStreet.getLatitude()+" "+lastFixOnStreet.getLongitude()+"\tSpeed "+speed+"\tStreet "+getStreetID()+"\t Provider "+location.getProvider()+"\t Correction "+lastFix.distanceTo(lastFixOnStreet));
		logMessage(msg, "FullFix");
	}
	
	private void updateSegment(Location location){
		if(!router.isRouteBeingCalculated())
		{
			lastSegmentLocation=location;
			mySegment=findSegment(location);
		}
	}
	
	public List<Location> getCurrentRoute(){
		if(router.isRouteCalculated())
		{
			return router.getCurrentRoute();
		}
		return null;
	}
	
	public boolean isOnStreet(){
		return mySegment!=null;
	}
	
	public boolean isRouteCalculated(){
		return router.isRouteCalculated();
	} 
	
	public long getStreetID(){
		if(mySegment!=null)
		{
			return mySegment.getRoad().getId();
		}
		return 0;
	}
	
	public float getDistanceToClosestIntersection(){
		Location joint=new Location("geoHelper");
		Location myLoc= getLocOnStreet();
		BinaryMapDataObject road = mySegment.getRoad();
		//Check the start node of my segment;
		joint.setLongitude(MapUtils.get31LongitudeX(road.getPoint31XTile(mySegment.getSegmentStart())));
		joint.setLatitude(MapUtils.get31LatitudeY(road.getPoint31YTile(mySegment.getSegmentStart())));
		
		float dist= myLoc.distanceTo(joint);
		
		joint.setLongitude(MapUtils.get31LongitudeX(road.getPoint31XTile(mySegment.getSegmentStart()+1)));
		joint.setLatitude(MapUtils.get31LatitudeY(road.getPoint31YTile(mySegment.getSegmentStart()+1)));
		float dist2=myLoc.distanceTo(joint);
		if( dist < dist2 ){
			return dist;
		}
		else{
			return dist2;
		}
	}
	
	/*
	 * Return the estimated location based on last fix and estimated speed+direction 
	 */
	public Location getLocation(){
		Location last=lastFix;
		Location now;
		if(last!=null && speed > 0.5)
		{
			long timeElapsed = System.currentTimeMillis()-last.getTime();
			float seconds=((float)timeElapsed)*((float)1e-3);
			float distance=seconds*speed;
			now= pointRadialDistance(last, bearing, distance);
		}
		else
		{
			now=last;
		}
		
		return now;
	}
	
	public Location getLocOnStreet(){
		Location last=lastFixOnStreet;
		Location now;
		if(last!=null && speedOnStr > 0.5)
		{
			long timeElapsed = System.currentTimeMillis()-last.getTime();
			float seconds=((float)timeElapsed)*((float)1e-3);
			float distance=seconds*speedOnStr;
			now= pointRadialDistance(last, bearingOnStr, distance);
		}
		else
		{
			now=last;
		}
		
		if(mySegment != null){
			int [] pos = posOnSegment(now, mySegment);
			now.setLatitude(MapUtils.get31LatitudeY(pos[1]));
			now.setLongitude(MapUtils.get31LongitudeX(pos[0]));
		}
		return now;
	}

	public RouteCalculationResult calculateRoute(Location loc1, Location loc2){
		RouteProvider provider=router.getRouter();
		LatLon loc2latLon = new LatLon(loc2.getLatitude(),loc2.getLongitude());
		RouteCalculationResult res = provider.calculateRouteImpl(loc1, loc2latLon, mode, RouteService.OSMAND, router.getContext(), null, false);
		return res;
	}
	
	public ApplicationMode getMode() {
		return mode;
	}

	public void setMode(ApplicationMode mode) {
		this.mode = mode;
	}
	
	/*
	 * Giving a distance a point and a bearing, calculate a new Location.
	 */
	public Location pointRadialDistance(Location loc, float bearing, float distance){
		double R=6371.01*1e3;
		Location locOut = new Location(loc);
		double rLat1=Math.toRadians(loc.getLatitude());
		double rLon1=Math.toRadians(loc.getLongitude());
		double rBearing= Math.toRadians(bearing);
		double rDistance=distance/R;
		
		double rLat=Math.asin( Math.sin(rLat1) * Math.cos(rDistance) + 
					Math.cos(rLat1) * Math.sin(rDistance) * Math.cos(rBearing) );
		double rLon=0;
		if (Math.abs(Math.cos(rLat))<0.000001) /*it's a pole*/
		{
			rLon=rLon1;
		}
		else
		{
			rLon= rLon1- Math.atan2( Math.sin(rBearing)*Math.sin(rDistance)*Math.cos(rLat1) 
							, Math.cos(rDistance)-Math.sin(rLat1)*Math.sin(rLat));
		}
		
		locOut.setLatitude(Math.toDegrees(rLat));
		locOut.setLongitude(Math.toDegrees(rLon));
		return locOut;
	}
	
	/*
	 * Devuelve el punto de la ruta más cercano, asi como la distancia al mismo
	 *  y la distancia en línea recta entre este y el punto de entrada.
	 */
	public Location getClosestFutureDistance(Location loc1, double[] result) throws RouteNotDefined{
		if(router.isRouteCalculated())
		{
			List<Location> route = router.getCurrentRoute();
			Location closest=route.get(0);
			Location previous=getLocation();
			double distance = 20020e3; //Distance between poles
			double roadDistance = 0;
			double totalDistance = 0;
			for(Location loc : route)
			{
				double distance2=loc1.distanceTo(loc);
				totalDistance += previous.distanceTo(loc);
				previous = loc;

				if( distance2 < distance){
					closest=loc1;
					distance= distance2;
					roadDistance = totalDistance;
				}
			}
			result[0] = roadDistance;
			result[1] = distance;
			return closest;
		}
		else
		{
			throw new RouteNotDefined();
		}
	}
	
	
	public void run(){
		Looper.prepare();
		location.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 0, this);
		location.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 0, 0, this);
		
		Date now = new Date(System.currentTimeMillis());
		java.text.DateFormat format = new SimpleDateFormat("d_kkmm");
		String date = format.format(now);
		String filename= Environment.getExternalStorageDirectory()+"/GeoHelper"+date+".log";
		try{
			pw = new PrintWriter(filename);
		}catch (FileNotFoundException e) {
		}
		
		Looper.loop();
	}
	
	private synchronized void logMessage(String msg, String name){
		if(pw!=null && LOGGING){
			pw.println(System.currentTimeMillis()+"\t"+name+"\t"+msg);
			pw.flush();
		}
	}
}
